@@ -126,14 +126,18 @@ def main():
 
         result, result_dict = model.predict(**predict_parameter)
         for eval_pos, eval_metric in enumerate(eval_metrics):
+            if len(result) < 1:
+                predicted = ""
             if 'QA' in model.__class__.__name__:
                 target = " ".join(input.split(" ")[int(target[0]): int(target[1])])
             if 'OneByOne' in model.__class__.__name__ and predict_parameter['beamsearch']:
-                predicted = result_dict['label_map'][eval_pos][0]
+                if len(result) > 0:
+                    predicted = result_dict['label_map'][eval_pos][0]
             elif 'Tagger' in model.__class__.__name__:
-                predicted = " ".join([list(d.values())[0] for d in result_dict['label_map']])
                 target = target.split(" ")
-                predicted = predicted.split(" ")
+                if len(result) > 0:
+                    predicted = " ".join([list(d.values())[0] for d in result_dict['label_map']])
+                    predicted = predicted.split(" ")
             else:
                 predicted = result[0] if len(result) > 0 else ''
 
