@@ -23,12 +23,21 @@ class loadQADataset(data.Dataset):
             with open(cache_path, "rb") as cf:
                 samples = pickle.load(cf)
         else:
+            total_data = 0
+            data_exceed_maxlen = 0
             for i in tqdm(get_data_from_file(fpath)):
                 tasks, task, input, target = i
                 feature = get_feature_from_data(tokenizer, input, target, maxlen=maxlen)
                 if len(feature['input']) <= maxlen and 0 <= feature['target'][0] < maxlen and 0 <= feature['target'][
                     1] < maxlen:
                     samples.append(feature)
+                else:
+                    data_exceed_maxlen += 1
+                total_data += 1
+
+            print("Processed " + str(total_data) + " data, removed " + str(
+                data_exceed_maxlen) + " data that exceed the maximum length.")
+
             if cache:
                 with open(cache_path, 'wb') as cf:
                     pickle.dump(samples, cf)
